window.symbolsize = 30;
window.lableFontsize = 10;

(function () {
/**
 *
 * @param {Object} param
 * {
 *  elt: 对应页面元素
 *  type: 对应的类型, 当有（多个案例option重复较多/option较为复杂）提出一个type
 * }
 * @param {Object} opt 生成echart的要替代option
 * @return {Object} chart
 */


const drawChart = (param = {}, opt = {}) => {
  const chart = echarts.init(param.elt)
  let option
  switch (param.type) {
    // 知识图谱
    case 'knowledge-graph':
    case 'knowledge-graph-en': {
      option = {
        title: {},
        animation: false,
        series: [
          {
            type: 'graph',
            roam: true,
            layout: 'force',
            symbolSize: window.symbolsize,
            label: {
              show: true,
              position: 'bottom',
              distance: 6,
              fontSize:window.lableFontsize,
              color: '#909AA2'
            },
            edgeLabel: false,
            edgeSymbol: ['circle', 'arrow'],
            edgeSymbolSize: [3, 6],
            lineStyle: {
              curveness: 0.2,
              color: '#B4C9DA'
            },
            emphasis: {
              itemStyle: {
                shadowBlur: 15,
                shadowOffsetY: 10
              }
            },
            scaleLimit: {
              min: 0.2 // 最小的缩放值
            },
            force: { 
              layoutAnimation:false 
            },
            roam:false,
            draggable:false,
          }
        ],
        tooltip: {
          trigger: 'item',
          position: 'bottom',
          backgroundColor: 'rgba(0,67,101,0.85)',
          padding: 10,
          formatter: function(data) {
            if (data.data.categoryName === '待检测') {
              return
            }
            let htmlStr = ''
            if (data.dataType === 'node') {
              htmlStr = `
                <div style="padding: 0.1rem;">
                  <div style="max-width:2.0rem;font-size: 0.28rem;color: #fff;overflow:hidden;text-overflow:ellipsis;white-space:nowrap;">${data.name}</div>
                  <div style="width:2.0rem;height:0.12rem;border-radius:0.2rem;background-color: #E2EFFA;margin:0.1rem 0 0.04rem;">
                    <div style="width:${data.value}%;background-color: ${data.color};height: 100%;border-radius:0.2rem;"></div>
                  </div>
                  <div style="font-size:0.2rem;color:rgba(255,255,255,1);">
                    <span style="margin-right: 0.28rem;color:rgba(255,255,255,.5);">掌握程度</span>
                    <span>${data.value}%</span>
                  </div>
                </div>
              `
            }
            return htmlStr
          }
        }
        
      }
      break
    }
    // pie
    case 'pie': {
      option = {
        series: [
          {
            type: 'pie'
          }
        ]
      }
      break
    }
    // radar
    case 'radar': {
      option = {
        series: [
          {
            type: 'radar'
          }
        ]
      }
      break
    }
    // line
    case 'line': {
      option = {
        series: [
          {
            type: 'line',
          }
        ]
      }
      break
    }
    default:
      break
  }
  // series设置, 优先级高
  option && Array.isArray(opt.series) &&
    opt.series.forEach((n, i) => {
      option.series[i] = Object.assign(option.series[i], n)
    })
  // 其他设置
  if (option) {
    option = Object.assign(option, opt, { series: option.series })
  } else {
    option = opt
  }
  chart.setOption(option)
  return chart
}

/**
 * 对外封装的方法, 简化调用
 * @param {Object}
 *  {
 *     type: {String} 必要
 *           对应约定好的对外封装类型，这个类型不一定要和drawChart的type一一对应
 *     elt: {Object} 必要
 *          对应页面元素
 *     data: {Array<Object>} 必要
 *     其他参数要求，看各自case的说明
 *  }
 * @return {Object} chart
 */
function draw(param = {}) {
  let chart = null
  switch (param.type) {
    /**
     * 知识图谱
     * repulsion: {Number} 可选, 默认1000
     * links: {Array<Object>} 必要
     *  [
     *   {
     *     source: {String} 对应data里的key
     *     target: {String} 同上
     *   }
     *  ]
     * categories: {Arnameray<Object>} 必要
     *  [
     *    {
     *      name: '未掌握',
     *      isAI: false
     *    },
     *    ...
     *  ]
     * data：示例
     *  [
     *   {
     *      name: {String} 对应名称
     *      value: {Number} 值
     *      category: {Number} 对应categories里的index
     *      symbolSize: {Number} 可选
     *   }
     *  ]
     */
    case 'knowledge-graph': {
      // 对应生成categories
      const categories = param.categories.map(n => {
        const obj = {
          symbol: 'circle',
          itemStyle: {
            color: '#B4C9DA'
          },
          emphasis: {
            itemStyle: {
              color: '#B4C9DA',
              shadowColor: 'transparent'
            }
          }
        }
        switch (n.name) {
          case '未掌握': {
            if (n.isAI) {
              obj.symbol = 'image://./img/kg_notmastered.png'
              obj.itemStyle.color = '#FFDFDC'
              obj.emphasis.itemStyle = {
                color: new echarts.graphic.LinearGradient(0, 0, 0, 1, [
                  {
                    offset: 0,
                    color: '#FFEBE6'
                  },
                  {
                    offset: 1,
                    color: '#FFC3BD'
                  }
                ]),
                shadowColor: '#FF9C92'
              }
            } else {
              obj.itemStyle.color = '#FF6051'
              obj.emphasis.itemStyle = {
                color: new echarts.graphic.LinearGradient(0, 0, 0, 1, [
                  {
                    offset: 0,
                    color: '#FF8579'
                  },
                  {
                    offset: 1,
                    color: '#FF3825'
                  }
                ]),
                shadowColor: '#FF9C92'
              }
            }
            break
          }
          case '部分掌握': {
            if (n.isAI) {
              obj.symbol = 'image://./img/kg_partialmastered.png'
              obj.itemStyle.color = '#FFF0CB'
              obj.emphasis.itemStyle = {
                color: new echarts.graphic.LinearGradient(0, 0, 0, 1, [
                  {
                    offset: 0,
                    color: '#FFFCF5'
                  },
                  {
                    offset: 1,
                    color: '#FFE8B0'
                  }
                ]),
                shadowColor: '#FFD165'
              }
            } else {
              obj.itemStyle.color = '#FFB400'
              obj.emphasis.itemStyle = {
                color: new echarts.graphic.LinearGradient(0, 0, 0, 1, [
                  {
                    offset: 0,
                    color: '#FFD56F'
                  },
                  {
                    offset: 1,
                    color: '#FFB400'
                  }
                ]),
                shadowColor: '#FFD165'
              }
            }
            break
          }
          // 完全掌握
          case '完全掌握': {
            if (n.isAI) {
              obj.symbol = 'image://./img/kg_fulllymastered.png'
              obj.itemStyle.color = '#15CF66'
              obj.emphasis.itemStyle = {
                color: new echarts.graphic.LinearGradient(0, 0, 0, 1, [
                  {
                    offset: 0,
                    color: '#E0FFEC'
                  },
                  {
                    offset: 1,
                    color: '#8DEDB7'
                  }
                ]),
                shadowColor: '#6DE19F'
              }
            } else {
              obj.itemStyle.color = '#15CF66'
              obj.emphasis.itemStyle = {
                color: new echarts.graphic.LinearGradient(0, 0, 0, 1, [
                  {
                    offset: 0,
                    color: '#00D4C1'
                  },
                  {
                    offset: 1,
                    color: '#15CF66'
                  }
                ]),
                shadowColor: '#6DE19F'
              }
            }
            break
          }
          // 默认待检测
          default: {
            break
          }
        }
        return obj
      })
      const data = param.data.map(n => {
        return {
          ...n,
          categoryName: param.categories[n.category].name
        }
      })
      chart = drawChart(param, {
        series: [
          {
            force: {
              repulsion: param.repulsion || 1000,
              layoutAnimation:false
              
            },
            categories,
            data,
            links: param.links
          }
        ]
      })
      break
    }
    case 'knowledge-graph-en': {
      chart = drawChart(param, {
        series: [
          {
            zoom: param.zoom || 1,
            force: {
              repulsion: param.repulsion || 1000,
               layoutAnimation:false
            },
            categories: param.categories,
            data: param.data,
            links: param.links
          }
        ]
      })
      break
    }
    /**
     * 知识掌握程度
     * data：示例
     *  [
     *   {
     *      name: {String} 对应名称
     *      value: {Number} 值
     *      color: {String} 颜色
     *   }
     *  ]
     */
    case 'knowledge-degree': {
      chart = drawChart(
        {
          type: 'pie',
          elt: param.elt
        },
        {
          color: param.data.map(n => n.color),
          legend: {
            show: false,
            data: ['未掌握', '部分掌握', '已掌握', '待检测']
          },
          series: [
            {
              hoverAnimation: false,
              radius: ['58%', '90%'],
              center: ['50%', '50%'],
              itemStyle: {
                borderWidth: 0
              },
              label: {
                show: false
              },
              labelLine: {
                show: false
              },
              data: param.data
            }
          ]
        }
      )
      break
    }
    /**
     * 练习正确率
     */
    case 'correct-rate': {
      chart = drawChart(
        {
          type: 'line',
          elt: param.elt
        },
        {
          grid: {
            top: 35,
            bottom: 30,
            right: 10,
            left: 40
          },
          xAxis: {
            type: 'category',
            axisLine: {
              lineStyle: {
                color: '#DDDFE7'
              }
            }
          },
          yAxis: {
            type: 'value',
            axisLine: {
              show: false,
              lineStyle: {
                color: '#DDDFE7'
              }
            },
            axisLabel: {
              formatter: '{value}%'
            },
            splitLine: {
              lineStyle: {
                color: '#DDDFE7'
              }
            },
            min: 0,
            max: 100,
            interval: 25
          },
          series: [{
            data: param.data,
            type: 'line',
            lineStyle: {
              color: '#336984'
            },
            symbol: 'circle',
            symbolSize: 6,
            itemStyle: {
              color: '#336984'
            },
            label: {
              show: true,
              backgroundColor: '#336984',
              color: '#fff',
              fontSize: 13,
              padding: [6, 5, 2, 5],
              borderRadius: [4, 4, 4, 4],
              formatter: '{c}%'
            }
          }]
        }
      )
      break
    }
    /**
     * 考点掌握情况
     */
    case 'knowledge-mastery-bar': {
      chart = drawChart(
        {
          elt: param.elt
        },
        {
          grid: {
            top: 25,
            bottom: 40,
            right: 0,
            left: 0
          },
          xAxis: [
            {
              type: 'category',
              data: param.xAxisText,
              nameTextStyle: {
                fontFamily: 'PingFang SC'
              },
              axisLabel: {
                textStyle: {
                  color: '#6D7278',
                  fontSize: 12
                }
              },
              axisTick: {
                show: false
              },
              axisLine: {
                lineStyle: {
                  color: '#DDDFE7'
                }
              }
            }
          ],
          yAxis: [
            {
              show: false,
              max: param.max
            }
          ],
          series: [
            {
              name: '本次',
              type: 'bar',
              data: param.data[0],
              barWidth: 28,
              barGap: 0,
              itemStyle: {
                normal: {
                  show: true,
                  color: param.barColor
                }
              },
              label: {
                color: '#004365',
                show: true,
                position: 'top',
                fontSize: 14,
                fontFamily: 'PingFang SC'
              }
            },
            {
              name: '上次对比',
              type: 'bar',
              data: param.data[1],
              barWidth: 28,
              barGap: 0,
              itemStyle: {
                normal: {
                  show: true,
                  color: '#DDDFE7'
                }
              },
              label: {
                color: '#004365',
                show: true,
                position: 'top',
                fontSize: 14,
                fontFamily: 'PingFang SC'
              }
            }
          ]
        }
      )
      break
    }
    /**
     * 能力模型三维图
     */
    case 'ability-radar': {
      chart = drawChart(
        {
          type: 'radar',
          elt: param.elt
        },
        {
          grid: {
            top: 300,
            bottom: 0,
            left: 0,
            right: 0
          },
          backgroundColor: '#fff',
          radar: {
            name: {
              textStyle: {
                color: '#6D7278',
                fontSize: 12
              }
            },
            indicator: [{
              name: '文言文',
              max: 100
            },
            {
              name: '古诗文',
              max: 100
            },
            {
              name: '语言基础',
              max: 100
            }
            ],
            axisLine: {
              lineStyle: {
                color: '#F2F3F6'
              }
            },
            splitLine: {
              lineStyle: {
                color: '#F2F3F6'
              }
            },
            splitArea: {
              areaStyle: {
                color: '#fff'
              }
            },
            splitNumber: 4
          },
          series: [{
            name: ' ',
            type: 'radar',
            symbol: 'none',
            data: [{
              value: param.data,
              name: '',
              lineStyle: {
                width: 0
              },
              areaStyle: {
                color: 'rgba(5, 198, 229, 0.3)'
              }
            }]
          }]
        }
      )
      break
    }
    default:
      break
  }
  return chart
}

window.draw = draw
})()
