(function () {

  const categoriesIndexMap = {
    '00': 6,
    '10': 2,
    '20': 1,
    '30': 0,
    '11': 2,
    '21': 1,
    '31': 0,
    '02': 6,
    '12': 5,
    '22': 4,
    '32': 3
  }

  const categories = [
    {
      name: '未掌握',
      isAI: false
    },
    {
      name: '部分掌握',
      isAI: false
    },
    {
      name: '完全掌握',
      isAI: false
    },
    {
      name: '未掌握',
      isAI: true
    },
    {
      name: '部分掌握',
      isAI: true
    },
    {
      name: '完全掌握',
      isAI: true
    },
    {
      name: '待检测',
      isAI: false
    }
  ]

  function createKnowledgesMap(relations) {
    const map = {}
    relations.forEach(item => {
      if (item.start.id && item.start.name) {
        map[item.start.id] = item.start.name
      }
      if (item.end.id && item.end.name) {
        map[item.end.id] = item.end.name
      }
    })
    return map
  }

  function createChartData(data) {
    const chartData = {
      data: [],
      links: []
    }
    const knowledgesMap = createKnowledgesMap(data.relations)

    data.knowledges.forEach(item => {
      chartData.data.push({
        name: knowledgesMap[item.knowledgeId] || '',
        value: Math.round(item.score * 100),
        category: categoriesIndexMap[item.level + '' + item.scoreType]
      })
    })
    data.relations.forEach(item => {
      if (item.relation === 1 && item.start && item.end) {
        chartData.links.push({
          source: item.end.name,
          target: item.start.name
        })
      } else if (item.relation === 2 && item.start && item.end) {
        chartData.links.push({
          source: item.start.name,
          target: item.end.name
        })
      }
    })
    return chartData
  }

  let knowledgeGraphChart



  function knowledgeGraphInit(data,theRepulsion,symbolsize,fontsize) {


    var repulsion = 800;
    if(typeof(repulsion) != 'undefined')
    {
      repulsion = theRepulsion;
    }
    if(typeof(symbolsize) != 'undefined' && typeof(fontsize) != 'undefined')
    {
      window.symbolsize = symbolsize;
      window.lableFontsize = fontsize;
    }else{
      var docEl = window.document.documentElement;
      var docWidth = docEl.getBoundingClientRect().width; 
      if(docWidth > 800)
      {
        repulsion = 800;
        window.symbolsize = 35;
        window.lableFontsize = 10;
         
      }else if(docWidth > 600)
      {
        repulsion = 400;
        window.symbolsize = 25;
        window.lableFontsize = 6;
         
      }else if(docWidth > 500)
      {
        repulsion = 240;
        window.symbolsize = 20;
        window.lableFontsize = 6;
          
      }
    }

   
   
    
 

    console.log('init data =====');
    console.log("repulsion "+repulsion +"   symbolsize:"+symbolsize + "   font-size:"+fontsize);
    console.log('data =====');
    console.log(data);
    const chartData = createChartData(data)
    console.log('chartData =====');
    console.log(chartData);
    knowledgeGraphChart = draw({
      type: 'knowledge-graph',
      elt: document.querySelector('#knowledgeGraph'),
      repulsion: repulsion,
      categories: categories,
      ...chartData
    })
    window.addEventListener('resize', knowledgeGraphChart.resize)
  }

  function knowledgeGraphInitBase64(base64Data,repulsion,symbolize,fontsize) {
    let data = Base64.decode(base64Data)
    data = JSON.parse(data)
    knowledgeGraphInit(data,repulsion,symbolize,fontsize);
  }

  function knowledgeGraphZoom(val) {
    const series = knowledgeGraphChart.getOption().series
    let zoomValue = 1
    if (series && series[0] && series[0].zoom) {
      zoomValue = series[0].zoom + val
    }
    zoomValue = zoomValue < 0.2 ? 0.2 : zoomValue
    zoomValue = zoomValue > 4 ? 4 : zoomValue
    knowledgeGraphChart.setOption({ series: [{ zoom: zoomValue }] })
  }

  window.knowledgeGraphInit = knowledgeGraphInit
  window.knowledgeGraphInitBase64 = knowledgeGraphInitBase64
  window.knowledgeGraphZoom = knowledgeGraphZoom

})()